from Products.CMFCore import CMFCorePermissions
from config import PROJECTNAME
from Products.Archetypes.public import *
from Products.CMFCore.utils import getToolByName
import csv
import os
from StringIO import StringIO

schema = Schema((
    StringField("import_type",
              languageIndependent = 1,
              schemata="Step1: choose type",
              vocabulary = "getTypes",
              widget = SelectionWidget(
						label = 'Portal type',
						description = 'Please select the portal type of the objects to import',
                        ),
              ),
   FileField("csv_file",
              languageIndependent = 1,
              schemata="Step2: upload file",
              allowable_content_types= ('text/comma-separated-values',),
              widget = RichWidget(
                        rows=15,
                        cols=50,          
                        label = 'CSV file',                        
                        description = 'Please upload a CSV file containing field values for the objects to create,\
                                       here you will see a preview of the content',
                        ),
              ),  
   LinesField("map_fields",
              languageIndependent = 1,
              schemata="Step3: map fields",
              widget = TextAreaWidget(
                        label = 'Map fields',
                        macro = 'widget_mapchooser',
                        description = 'Please map csv to AT fields',
                        ),
              ),  
   BooleanField("update",
              schemata="Step3: map fields",
              default=0,
              widget = BooleanWidget(
                        label = 'Update',                        
                        description = 'Select this check box if you would like to enable updating. \
                                       With this enabled, any objects that already exist will be updated\
                                       with the provided data.',
                        ),
              ),                              
   StringField("prefix",
              languageIndependent = 1,
              schemata="Step3: map fields",
              default="imported",
              widget = StringWidget(
                        label = 'prefix',                        
                        description = 'Please choose a prefix for the id of the imported objects, \
                                       the ids will be generated like this: imported.2005-11-10.113647',
                        ),
              ),                
   LinesField("start_import",
              languageIndependent = 1,
              schemata="Step4: start batch import",
              widget = TextAreaWidget(
                        label = 'Start batch import',
                        macro = 'widget_startimport',
                        ),
              ),
                 ))
  


schema = BaseSchema + schema

class Importer(BaseContent):
    
    __implements__ = BaseContent.__implements__
    
    def getTypes(self):
        at_tool=getToolByName(self,'archetype_tool')
        at_types=at_tool.listRegisteredTypes()        
        at_list=[]
        for at in at_types:
            at_list.append('%s.%s.%s' % (at['package'],at['meta_type'],at['portal_type']))
        return at_list



    schema = schema
    archetype_name = "Importer"
    immediate_view = "importer_edit"
    
    actions = (
        { 'id': 'view',
            'name': 'View',
            'action': 'string:${object_url}/importer_edit',
            'permissions': (CMFCorePermissions.View,),
            'category':'object',
            },
        { 'id': 'edit',
            'name': 'Edit',
            'action': 'string:${object_url}/importer_edit',
            'permissions': (CMFCorePermissions.View,),
            'category':'object',
            },
        
          )
 
 
    def getCSVheader(self):
        data=self.getCSVdata()
        return self.head
    
    
    def getCSVdata(self):
        file_upload=self.getCsv_file()
        filename = file_upload.filename 

        stringio_file = StringIO(str(file_upload))
        filedatas = stringio_file.read()

        # get informations about the file
        snif = csv.Sniffer()        
        #if not snif.has_header(filedatas):
            #raise 'You must add header to the csv file!'

        #get the header
        rdr = csv.reader(StringIO(filedatas), snif.sniff(filedatas))
        header = rdr.next() # assume first row is header
        self.head=header

        dialect = snif.sniff(filedatas)

        reader = csv.DictReader(StringIO(filedatas),fieldnames=header,dialect=dialect) 
        stringio_file.close() 
        return list(reader)
    

registerType(Importer, PROJECTNAME)
